// Copyright (C) 1996 by Florian Raemy & LCAV, Swiss Federal Institute of 
// Technology, Lausanne, Switzerland.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//
// Visit Florian's Home Page http://studwww.eurecom.fr/~raemy and
// LCAV Department of EPFL: http://lcavwww.epfl.ch

import java.awt.*;
import java.io.*;
import java.util.*;


/***
* Hufmann - A java implementation of the Hufmann coding and decoding algorithm
***/


public class Hufmann
{
	public int ImageHeight;
	public int ImageWidth;
	public int DC_matrix[][];
    public int AC_matrix[][][];
    public int code;
    public int mat[][];

    /*
    * The constructor of the class Hufmann
    * @param  code:     This correspond the luminance and chrominance Hufmann tables. 1=luminance, 0=chrominance
    * @param  Wid:      The width of the image
    * @param  Hei:      The height of the image
    */
	public Hufmann(int code,int Wid,int Hei)
	{

	    init_hufcode(code);
	    code=code;
	    ImageWidth=Wid;
	    ImageHeight=Hei;

	}

    /*
    * This method gives the size of the AC value. Depending on the range of value of the input parameter
    * the size will be different.( example: -15..-8;8..15, size=4)
    * @param value:     The value for which you search the size
    * @return size:     The size of value according to the Jpeg standart
    */

   public int Get_AC_Size(int value)
   {
        int size;
        int i;
        int exp;
        size=0;
        exp=1;
        for(i=0;i<12;i++)
        {


            if (Math.abs(value)<exp)
            {
                size=i;

                i=12;
            }
            exp*=2;
        }
        return size;
   }

    /*
    * This method gives the size of the DC value. Depending on the range of value of the input parameter
    * the size will be different.( example: -15..-8;8..15, size=4)
    * @param value:     The value for which you search the size
    * @return size:     The size of value according to the Jpeg standart
    */
    public int Get_DC_Size(int value)
    {
        int size;
        int i;
        size=0;
        int exp;
        exp=1;
        for(i=0;i<11;i++)
        {

            if (Math.abs(value)<exp)
            {
                size=i;

                i=11;
            }
            exp*=2;
        }
        return size;
    }



    /*
    * The Hufmann tables defines for a given size and number of zero, the hufmann code and the number of bit to
    * append with the hufmann code. So, for a specific value, according to the number of bit which has to be append
    * to the code, this method calculate the value.
    * @param value:         The value you want to encode
    * @param nbre_of_bit    The number of bit you need to encode it
    * @return output        The integer value which represents the number encoded on "nbre_of_bit" bits
    */
    public int Give_Follow(int value,int nbre_of_bit)//marche
    {
        int coded_int;
        int tmp;
        int shift;
        shift=2048;
        int output;
        tmp=0;

        /*
        * If the number is negative, a shift for encoding is needed
        */
        if (value<0)
        {
            coded_int=value+shift-1;
        }
        else
        {
            coded_int=value+shift;
        }

        if (nbre_of_bit==1)
        {
            tmp=0x1;
        }
        if (nbre_of_bit==2)
        {
            tmp=0x3;
        }
        if (nbre_of_bit==3)
        {
            tmp=0x7;
        }
        if (nbre_of_bit==4)
        {
            tmp=0xf;
        }
        if (nbre_of_bit==5)
        {
            tmp=0x1f;
        }
        if (nbre_of_bit==6)
        {
            tmp=0x3f;
        }
        if (nbre_of_bit==7)
        {
            tmp=0x7f;
        }
        if (nbre_of_bit==8)
        {
            tmp=0xff;
        }
        if (nbre_of_bit==9)
        {
            tmp=0x1ff;
        }
        if (nbre_of_bit==10)
        {
            tmp=0x3ff;
        }
        if (nbre_of_bit==11)
        {
            tmp=0x7ff;
        }
        if (nbre_of_bit==12)
        {
            tmp=0xfff;
        }
        if (nbre_of_bit==13)
        {
            tmp=0x1fff;
        }
        if (nbre_of_bit==14)
        {
            tmp=0x3fff;
        }
        if (nbre_of_bit==15)
        {
            tmp=0x7fff;
        }
        if (nbre_of_bit==16)
        {
            tmp=0xffff;
        }
        output=(tmp & coded_int);

        return output;
    }

    /*
    * The Hufmann tables defines for a given size and number of zero, the hufmann code and the number of bit to
    * append with the hufmann code. So, for a specific encoded value, according to the number of bit which is append
    * to the code, this method return the integer of the original value.
    * @param value:         The value you want to decode
    * @param nbre_of_bit    The number of bit you need to decode it
    * @return output        The integer value which represents the number decoded on "nbre_of_bit" bits
    */

    public int Give_Back_Follow(int value,int nbre_of_bit)//marche
    {
        int coded_int;
        int tmp;
        int shift;
        shift=2048;
        int output;
        tmp=0;

        if (nbre_of_bit==1)
        {
            tmp=0xfffffffe;
        }
        if (nbre_of_bit==2)
        {
            tmp=0xfffffffc;
        }
        if (nbre_of_bit==3)
        {
            tmp=0xfffffff8;
        }
        if (nbre_of_bit==4)
        {
            tmp=0xfffffff0;
        }
        if (nbre_of_bit==5)
        {
            tmp=0xffffffe0;
        }
        if (nbre_of_bit==6)
        {
            tmp=0xffffffc0;
        }
        if (nbre_of_bit==7)
        {
            tmp=0xffffff80;
        }
        if (nbre_of_bit==8)
        {
            tmp=0xffffff00;
        }
        if (nbre_of_bit==9)
        {
            tmp=0xfffffe00;
        }
        if (nbre_of_bit==10)
        {
            tmp=0xfffffc00;
        }
        if (nbre_of_bit==11)
        {
            tmp=0xfffff800;
        }
        if (nbre_of_bit==12)
        {
            tmp=0xfffff000;
        }
        if (nbre_of_bit==13)
        {
            tmp=0xffffe000;
        }
        if (nbre_of_bit==14)
        {
            tmp=0xffffc000;
        }
        if (nbre_of_bit==15)
        {
            tmp=0xffff8000;
        }
        if (nbre_of_bit==16)
        {
            tmp=0xffff0000;
        }

        /*
        * Used to know if there this is positive or negative
        */
        coded_int=((value) >> (nbre_of_bit-1));
        if (coded_int>0)
        {
            output=value;
        }
        else
        {
            output=(tmp | (value+1));
        }
        if (nbre_of_bit==0)
        {
            output=0;
        }
        return output;
    }


    /*
    * The method gives the AC hufmann code for a given number of zero and size
    * @param size:      The size of the value (cf Get_AC_size)
    * @param nbre_zero: The nuber of zero given by the RLE encoding
    * @return code:     The hufmann AC code
    */
    public int[] Get_AC_Code(int size,int nbre_zero)
    {
        int code[];
        code=new int[2];
        code[0]=AC_matrix[nbre_zero][size][0]; // this is the hufmann code
        code[1]=AC_matrix[nbre_zero][size][1]; // this is the number on which data is encoded
        return code;
    }


    /*
    * The method gives the DC hufmann code for a given number of zero and size
    * @param size:      The size of the value (cf Get_DC_size)
    * @param nbre_zero: The nuber of zero given by the RLE encoding
    * @return code:     The hufmann DC code
    */
    public int[] Get_DC_Code(int size)
    {
        int code[];
        code=new int[2];
        code[0]=DC_matrix[size][0]; // this is the hufmann code
        code[1]=DC_matrix[size][1]; // this is the number of bits on which data is encoded
        return code;
    }


    /*
    * Give_AC_Code gives you the size and number of zero when you gives the AC code
    * @param input:     hufcode is the AC code of the data
    * @return output:    output[0] corresponds to the size of the data (=number of bit appended)
    *                   output[1] corresponds to the number of zero
    */
    public int[] Give_AC_Code(int hufcode)
    {
        int i,j;
        int output[];
        output=new int[2];

        for (i=0;i<16;i++)
        {
            for(j=0;j<11;j++)
            {
                if(AC_matrix[i][j][0]==hufcode)
                {
                    output[1]=i;//number of zero
                    output[0]=j;// size
                    i=16;
                    j=11;
                }
            }
        }
        return output;
    }


    /*
    * Give_DC_Code gives you size when you gives the DC code
    * @param input:      hufcode is the DC code of the data
    * @return output:    output corresponds to the size of the data (=number of bit appended)
    */
    public int Give_DC_Code(int hufcode)
    {
        int i;
        int output;
        output=0;
        for(i=0;i<12;i++)
        {
            if (hufcode==DC_matrix[i][0])
            {
                output=i;
                i=12;
            }
        }
        return output;
    }


    /*
    * Hufmann_encode_64 provides the full hufmann coding of a 8x8 block
    * @param s:         A stack of RLEdata. It is composed by the number of zero and amplitude of the 8x8 block
    *                   the first on the top is the last AC coefficient and the last is the DC.
                        The last value on the stack is the number of bits to encode the 8x8 block
    * @return output:   A stack of Hufmann data. It is composed by the hufmann code and the bits to append to the code.
    *                    The data on the top is the DC coefficient and the last is the last AC
    */
    public Stack Hufmann_encode_64(Stack s)
    {
        RLEdata value;
        Object obj;
        Hufmanndata hufd;
        Stack out;


        int amplitude;
        int nbre_zero;
        int size;
        int calc[];
        int follow;
        int code;
        int bits;
        calc=new int[2];
        out= new Stack();
        bits=0;
        while(!s.empty())
	    {
	        obj=s.pop();

	        value=(RLEdata)obj;

	        /*
	        * Calculate the DC component: we have here amplitude and number of zero
	        */
	        if (s.empty())
	        {
	            amplitude=value.amplitude();
	            nbre_zero=value.number_zero();
	            size=Get_DC_Size(amplitude);
	            calc=Get_DC_Code(size);
	            follow=Give_Follow(amplitude,size);
	            code=calc[0];
	            bits=bits+calc[1]+size;//this is the nuber of bits needed to encode the 8x8 block
	        }

	        /*
	        * Calculate the AC components: we have here amplitude and number of zero
	        */
	        else
	        {
	            amplitude=value.amplitude();
	            nbre_zero=value.number_zero();
	            size=Get_AC_Size(amplitude);
	            calc=Get_AC_Code(size,nbre_zero);
	            follow=Give_Follow(amplitude,size);
	            code=calc[0];
	            bits=bits+calc[1]+size;//this is the nuber of bits needed to encode the 8x8 block
	        }
	        hufd=new Hufmanndata(code,follow);
	        out.push(hufd);
	    }
	    hufd=new Hufmanndata(bits,0);
	    out.push(hufd);
        return out;
    }

    /*
    * Hufmann_decode_64 provides the full hufmann decoding of a 8x8 block
    * @param s:          A stack of Hufmann data. It is composed by the hufmann code and the bits to append to the code.
    *                    The data on the top is the DC coefficient and the last is the last AC
    * @return output:   A stack of RLEdata. It is composed by the number of zero and amplitude of the 8x8 block
    *                   the first on the top is the last AC coefficient and the last is the DC.
    */
    public Stack Hufmann_decode_64(Stack s)
    {
        RLEdata value;
        Object obj;
        Hufmanndata hufd;
        Stack out;
        int append_bit;
        int huf_code;
        int amplitude;
        int nbre_zero;
        int size;
        int calc[];
        int follow;
        int comp=0;
        calc=new int[2];
        out= new Stack();

        while(!s.empty())
	    {

	        obj=s.pop();
	        hufd=(Hufmanndata)obj;
	        if (comp==0)
	        {
	            huf_code=hufd.hufcode();
	            append_bit=hufd.appendbit();
	            size=Give_DC_Code(huf_code);
	            amplitude=Give_Back_Follow(append_bit,size);
	            value= new RLEdata(0,amplitude);
	            comp=comp+1;
	        }
	        else
	        {
	            huf_code=hufd.hufcode();
	            append_bit=hufd.appendbit();
	            calc=Give_AC_Code(huf_code);
	            amplitude=Give_Back_Follow(append_bit,calc[0]);
	            value= new RLEdata(calc[1],amplitude);
	        }
	        out.push(value);
	    }
	    return out;
	}

    /*
    * Hufmann_encode makes a Hufmann coding on a full scan. It uses Hufmann_encode_64.
    * It makes a Hufmann coding on the value which are already coded by RLE
    * We always take the first value of the stack off, because it is the number of bits to encode a 8x8 block
    * @param s[]:   An array of RLEdata(nbre_of_zero&amplitude) Stack. The disposition of the vector is this. s[0] is the left-top 8x8 block
    *               and s[ImageWidth*ImageHeight/8 -1] is the right-down 8x8 block of the scan.
    * @return t[]:  An array of Hufmanndata(hufmann code&value to append) Stack. The disposition is the same as before.
    */
    public Stack[] Hufmann_encode(Stack s[])
	{
	    Stack[] t;
	    int max;
	    max=ImageWidth*ImageHeight/64;
	    t=new Stack[max];
	    int i;
	    Object obj;
	    int image_len;
	    Hufmanndata hufd;
	    image_len=0;
        System.out.println("Hufmann Coding");
        for(i=0;i<max;i++)
	    {

	        t[i]=Hufmann_encode_64(s[i]);
	        obj=t[i].pop();
	        hufd=(Hufmanndata)obj;
	        image_len=image_len+hufd.hufcode();
        }
        System.out.println(" compression rate = "+(float)(ImageWidth*ImageHeight*8)/image_len);
	    return t;
	}

    /*
    * Hufmann_decode makes a Hufmann decoding on a full scan. It uses Hufmann_decode_64.
    * It makes a Hufmann decoding on the value which are already coded by Hufmann_encode and gives the RLE encoded matrix
    * @param s[]:   An array of  Hufmanndata(hufmann code&value to append)Stack. The disposition of the vector is this.
    *               s[0] is the left-top 8x8 block and s[ImageWidth*ImageHeight/8 -1] is the right-down 8x8 block of the scan.
    * @return t[]:  An array of RLEdata(nbre_of_zero&amplitude) Stack. The disposition is the same as before.
    */
	public Stack[] Hufmann_decode(Stack s[])
	{
        Stack[] t;
	    int max;
	    max=ImageWidth*ImageHeight/64;
	    t=new Stack[max];
	    int i;
        System.out.println("Hufmann Decoding");

        for(i=0;i<max;i++)
	    {
	        t[i]=Hufmann_decode_64(s[i]);
        }
	    return t;
	}

    /*
    * Initialisation of the differents Hufmann codes for Chrominance and Luminance elements
    * The tables are defined by experience according to the Jpeg standart (CCIT RECOMMENDATION T.81
    *  & ISO DIS 109181-1)
    */
    public void init_hufcode(int code)
    {
        DC_matrix=new int[12][2];
        AC_matrix=new int[16][11][2];
        mat=new int[5][2];

        /*
        * INIT OF CHROMINANCE
        */
        if(code==0)
        {

        /*
        * init of the DC values for the chrominance
        * [][0] is the code   [][1] is the number of bit
        */
        DC_matrix[0][0]=0x00;
        DC_matrix[0][1]=2;
        DC_matrix[1][0]=0x02;
        DC_matrix[1][1]=3;
        DC_matrix[2][0]=0x03;
        DC_matrix[2][1]=3;
        DC_matrix[3][0]=0x04;
        DC_matrix[3][1]=3;
        DC_matrix[4][0]=0x05;
        DC_matrix[4][1]=3;
        DC_matrix[5][0]=0x6;
        DC_matrix[5][1]=3;
        DC_matrix[6][0]=0xe;
        DC_matrix[6][1]=4;
        DC_matrix[7][0]=0x1e;
        DC_matrix[7][1]=5;
        DC_matrix[8][0]=0x3e;
        DC_matrix[8][1]=6;
        DC_matrix[9][0]=0x7e;
        DC_matrix[9][1]=7;
        DC_matrix[10][0]=0xfe;
        DC_matrix[10][1]=8;
        DC_matrix[11][0]=0x1fe;
        DC_matrix[11][1]=9;


        /*
        * Init of the AC hufmann code for the chrominance
        * matrix [][][0] is the code & matrix[][][1] is the number of bit needed
        */
        AC_matrix[0][0][0]=0x0;
        AC_matrix[0][0][1]=2;
        AC_matrix[0][1][0]=0x1;
        AC_matrix[0][1][1]=2;
        AC_matrix[0][2][0]=0x4;
        AC_matrix[0][2][1]=3;
        AC_matrix[0][3][0]=0xa;
        AC_matrix[0][3][1]=4;
        AC_matrix[0][4][0]=0x18;
        AC_matrix[0][4][1]=5;
        AC_matrix[0][5][0]=0x19;
        AC_matrix[0][5][1]=5;
        AC_matrix[0][6][0]=0x38;
        AC_matrix[0][6][1]=6;
        AC_matrix[0][7][0]=0x78;
        AC_matrix[0][7][1]=7;
        AC_matrix[0][8][0]=0x1f4;
        AC_matrix[0][8][1]=9;
        AC_matrix[0][9][0]=0x3f6;
        AC_matrix[0][9][1]=10;
        AC_matrix[0][10][0]=0xff4;
        AC_matrix[0][10][1]=12;
        AC_matrix[1][1][0]=0xb;
        AC_matrix[1][1][1]=4;
        AC_matrix[1][2][0]=0x39;
        AC_matrix[1][2][1]=6;
        AC_matrix[1][3][0]=0xf6;
        AC_matrix[1][3][1]=8;
        AC_matrix[1][4][0]=0x1f5;
        AC_matrix[1][4][1]=9;
        AC_matrix[1][5][0]=0x7f6;
        AC_matrix[1][5][1]=11;
        AC_matrix[1][6][0]=0xff5;
        AC_matrix[1][6][1]=12;
        AC_matrix[1][7][0]=0xff88;
        AC_matrix[1][7][1]=16;
        AC_matrix[1][8][0]=0xff89;
        AC_matrix[1][8][1]=16;
        AC_matrix[1][9][0]=0xff8a;
        AC_matrix[1][9][1]=16;
        AC_matrix[1][10][0]=0xff8b;
        AC_matrix[1][10][1]=16;
        AC_matrix[2][1][0]=0x1a;
        AC_matrix[2][1][1]=5;
        AC_matrix[2][2][0]=0xf7;
        AC_matrix[2][2][1]=8;
        AC_matrix[2][3][0]=0x3f7;
        AC_matrix[2][3][1]=10;
        AC_matrix[2][4][0]=0xff6;
        AC_matrix[2][4][1]=12;
        AC_matrix[2][5][0]=0x7fc2;
        AC_matrix[2][5][1]=15;
        AC_matrix[2][6][0]=0xff8c;
        AC_matrix[2][6][1]=16;
        AC_matrix[2][7][0]=0xff8d;
        AC_matrix[2][7][1]=16;
        AC_matrix[2][8][0]=0xff8e;
        AC_matrix[2][8][1]=16;
        AC_matrix[2][9][0]=0xff8f;
        AC_matrix[2][9][1]=16;
        AC_matrix[2][10][0]=0xff90;
        AC_matrix[2][10][1]=16;
        AC_matrix[3][1][0]=0x1b;
        AC_matrix[3][1][1]=5;
        AC_matrix[3][2][0]=0xf8;
        AC_matrix[3][2][1]=8;
        AC_matrix[3][3][0]=0x3f8;
        AC_matrix[3][3][1]=10;
        AC_matrix[3][4][0]=0xff7;
        AC_matrix[3][4][1]=12;
        AC_matrix[3][5][0]=0xff91;
        AC_matrix[3][5][1]=16;
        AC_matrix[3][6][0]=0xff92;
        AC_matrix[3][6][1]=16;
        AC_matrix[3][7][0]=0xff93;
        AC_matrix[3][7][1]=16;
        AC_matrix[3][8][0]=0xff94;
        AC_matrix[3][8][1]=16;
        AC_matrix[3][9][0]=0xff95;
        AC_matrix[3][9][1]=16;
        AC_matrix[3][10][0]=0xff96;
        AC_matrix[3][10][1]=16;
        AC_matrix[4][1][0]=0x3a;
        AC_matrix[4][1][1]=6;
        AC_matrix[4][2][0]=0x1f6;
        AC_matrix[4][2][1]=9;
        AC_matrix[4][3][0]=0xff97;
        AC_matrix[4][3][1]=16;
        AC_matrix[4][4][0]=0xff98;
        AC_matrix[4][4][1]=16;
        AC_matrix[4][5][0]=0xff99;
        AC_matrix[4][5][1]=16;
        AC_matrix[4][6][0]=0xff9a;
        AC_matrix[4][6][1]=16;
        AC_matrix[4][7][0]=0xff9b;
        AC_matrix[4][7][1]=16;
        AC_matrix[4][8][0]=0xff9c;
        AC_matrix[4][8][1]=16;
        AC_matrix[4][9][0]=0xff9d;
        AC_matrix[4][9][1]=16;
        AC_matrix[4][10][0]=0xff9e;
        AC_matrix[4][10][1]=16;
        AC_matrix[5][1][0]=0x3b;
        AC_matrix[5][1][1]=6;
        AC_matrix[5][2][0]=0x3f9;
        AC_matrix[5][2][1]=10;
        AC_matrix[5][3][0]=0xff9f;
        AC_matrix[5][3][1]=16;
        AC_matrix[5][4][0]=0xffa0;
        AC_matrix[5][4][1]=16;
        AC_matrix[5][5][0]=0xffa1;
        AC_matrix[5][5][1]=16;
        AC_matrix[5][6][0]=0xffa2;
        AC_matrix[5][6][1]=16;
        AC_matrix[5][7][0]=0xffa3;
        AC_matrix[5][7][1]=16;
        AC_matrix[5][8][0]=0xffa4;
        AC_matrix[5][8][1]=16;
        AC_matrix[5][9][0]=0xffa5;
        AC_matrix[5][9][1]=16;
        AC_matrix[5][10][0]=0xffa6;
        AC_matrix[5][10][1]=16;
        AC_matrix[6][1][0]=0x79;
        AC_matrix[6][1][1]=7;
        AC_matrix[6][2][0]=0x7f7;
        AC_matrix[6][2][1]=11;
        AC_matrix[6][3][0]=0xffa7;
        AC_matrix[6][3][1]=16;
        AC_matrix[6][4][0]=0xffa8;
        AC_matrix[6][4][1]=16;
        AC_matrix[6][5][0]=0xffa9;
        AC_matrix[6][5][1]=16;
        AC_matrix[6][6][0]=0xffaa;
        AC_matrix[6][6][1]=16;
        AC_matrix[6][7][0]=0xffab;
        AC_matrix[6][7][1]=16;
        AC_matrix[6][8][0]=0xffac;
        AC_matrix[6][8][1]=16;
        AC_matrix[6][9][0]=0xffad;
        AC_matrix[6][9][1]=16;
        AC_matrix[6][10][0]=0xffae;
        AC_matrix[6][10][1]=16;
        AC_matrix[7][1][0]=0x7a;
        AC_matrix[7][1][1]=7;
        AC_matrix[7][2][0]=0x7f8;
        AC_matrix[7][2][1]=11;
        AC_matrix[7][3][0]=0xffaf;
        AC_matrix[7][3][1]=16;
        AC_matrix[7][4][0]=0xffb0;
        AC_matrix[7][4][1]=16;
        AC_matrix[7][5][0]=0xffb1;
        AC_matrix[7][5][1]=16;
        AC_matrix[7][6][0]=0xffb2;
        AC_matrix[7][6][1]=16;
        AC_matrix[7][7][0]=0xffb3;
        AC_matrix[7][7][1]=16;
        AC_matrix[7][8][0]=0xffb4;
        AC_matrix[7][8][1]=16;
        AC_matrix[7][9][0]=0xffb5;
        AC_matrix[7][9][1]=16;
        AC_matrix[7][10][0]=0xffb6;
        AC_matrix[7][10][1]=16;
        AC_matrix[8][1][0]=0xf9;
        AC_matrix[8][1][1]=8;
        AC_matrix[8][2][0]=0xffb7;
        AC_matrix[8][2][1]=16;
        AC_matrix[8][3][0]=0xffb8;
        AC_matrix[8][3][1]=16;
        AC_matrix[8][4][0]=0xffb9;
        AC_matrix[8][4][1]=16;
        AC_matrix[8][5][0]=0xffba;
        AC_matrix[8][5][1]=16;
        AC_matrix[8][6][0]=0xffbb;
        AC_matrix[8][6][1]=16;
        AC_matrix[8][7][0]=0xffbc;
        AC_matrix[8][7][1]=16;
        AC_matrix[8][8][0]=0xffbd;
        AC_matrix[8][8][1]=16;
        AC_matrix[8][9][0]=0xffbe;
        AC_matrix[8][9][1]=16;
        AC_matrix[8][10][0]=0xffbf;
        AC_matrix[8][10][1]=16;
        AC_matrix[9][1][0]=0x1f7;
        AC_matrix[9][1][1]=9;
        AC_matrix[9][2][0]=0xffc0;
        AC_matrix[9][2][1]=16;
        AC_matrix[9][3][0]=0xffc1;
        AC_matrix[9][3][1]=16;
        AC_matrix[9][4][0]=0xffc2;
        AC_matrix[9][4][1]=16;
        AC_matrix[9][5][0]=0xffc3;
        AC_matrix[9][5][1]=16;
        AC_matrix[9][6][0]=0xffc4;
        AC_matrix[9][6][1]=16;
        AC_matrix[9][7][0]=0xffc5;
        AC_matrix[9][7][1]=16;
        AC_matrix[9][8][0]=0xffc6;
        AC_matrix[9][8][1]=16;
        AC_matrix[9][9][0]=0xffc7;
        AC_matrix[9][9][1]=16;
        AC_matrix[9][10][0]=0xffc8;
        AC_matrix[9][10][1]=16;
        AC_matrix[10][1][0]=0x1f8;
        AC_matrix[10][1][1]=9;
        AC_matrix[10][2][0]=0xffc9;
        AC_matrix[10][2][1]=16;
        AC_matrix[10][3][0]=0xffca;
        AC_matrix[10][3][1]=16;
        AC_matrix[10][4][0]=0xffcb;
        AC_matrix[10][4][1]=16;
        AC_matrix[10][5][0]=0xffcc;
        AC_matrix[10][5][1]=16;
        AC_matrix[10][6][0]=0xffcd;
        AC_matrix[10][6][1]=16;
        AC_matrix[10][7][0]=0xffce;
        AC_matrix[10][7][1]=16;
        AC_matrix[10][8][0]=0xffcf;
        AC_matrix[10][8][1]=16;
        AC_matrix[10][9][0]=0xffd0;
        AC_matrix[10][9][1]=16;
        AC_matrix[10][10][0]=0xffd1;
        AC_matrix[10][10][1]=16;
        AC_matrix[11][1][0]=0x1f9;
        AC_matrix[11][1][1]=9;
        AC_matrix[11][2][0]=0xffd2;
        AC_matrix[11][2][1]=16;
        AC_matrix[11][3][0]=0xffd3;
        AC_matrix[11][3][1]=16;
        AC_matrix[11][4][0]=0xffd4;
        AC_matrix[11][4][1]=16;
        AC_matrix[11][5][0]=0xffd5;
        AC_matrix[11][5][1]=16;
        AC_matrix[11][6][0]=0xffd6;
        AC_matrix[11][6][1]=16;
        AC_matrix[11][7][0]=0xffd7;
        AC_matrix[11][7][1]=16;
        AC_matrix[11][8][0]=0xffd8;
        AC_matrix[11][8][1]=16;
        AC_matrix[11][9][0]=0xffd9;
        AC_matrix[11][9][1]=16;
        AC_matrix[11][10][0]=0xffda;
        AC_matrix[11][10][1]=16;
        AC_matrix[12][1][0]=0x1fa;
        AC_matrix[12][1][1]=9;
        AC_matrix[12][2][0]=0xffdb;
        AC_matrix[12][2][1]=16;
        AC_matrix[12][3][0]=0xffdc;
        AC_matrix[12][3][1]=16;
        AC_matrix[12][4][0]=0xffdd;
        AC_matrix[12][4][1]=16;
        AC_matrix[12][5][0]=0xffde;
        AC_matrix[12][5][1]=16;
        AC_matrix[12][6][0]=0xffdf;
        AC_matrix[12][6][1]=16;
        AC_matrix[12][7][0]=0xffe0;
        AC_matrix[12][7][1]=16;
        AC_matrix[12][8][0]=0xffe1;
        AC_matrix[12][8][1]=16;
        AC_matrix[12][9][0]=0xffe2;
        AC_matrix[12][9][1]=16;
        AC_matrix[12][10][0]=0xffe3;
        AC_matrix[12][10][1]=16;
        AC_matrix[13][1][0]=0x7f9;
        AC_matrix[13][1][1]=11;
        AC_matrix[13][2][0]=0xffe4;
        AC_matrix[13][2][1]=16;
        AC_matrix[13][3][0]=0xffe5;
        AC_matrix[13][3][1]=16;
        AC_matrix[13][4][0]=0xffe6;
        AC_matrix[13][4][1]=16;
        AC_matrix[13][5][0]=0xffe7;
        AC_matrix[13][5][1]=16;
        AC_matrix[13][6][0]=0xffe8;
        AC_matrix[13][6][1]=16;
        AC_matrix[13][7][0]=0xffe9;
        AC_matrix[13][7][1]=16;
        AC_matrix[13][8][0]=0xffea;
        AC_matrix[13][8][1]=16;
        AC_matrix[13][9][0]=0xffeb;
        AC_matrix[13][9][1]=16;
        AC_matrix[13][10][0]=0xffec;
        AC_matrix[13][10][1]=16;
        AC_matrix[14][1][0]=0x3fe0;
        AC_matrix[14][1][1]=14;
        AC_matrix[14][2][0]=0xffed;
        AC_matrix[14][2][1]=16;
        AC_matrix[14][3][0]=0xffee;
        AC_matrix[14][3][1]=16;
        AC_matrix[14][4][0]=0xffef;
        AC_matrix[14][4][1]=16;
        AC_matrix[14][5][0]=0xfff0;
        AC_matrix[14][5][1]=16;
        AC_matrix[14][6][0]=0xfff1;
        AC_matrix[14][6][1]=16;
        AC_matrix[14][7][0]=0xfff2;
        AC_matrix[14][7][1]=16;
        AC_matrix[14][8][0]=0xfff3;
        AC_matrix[14][8][1]=16;
        AC_matrix[14][9][0]=0xfff4;
        AC_matrix[14][9][1]=16;
        AC_matrix[14][10][0]=0xfff5;
        AC_matrix[14][10][1]=16;
        AC_matrix[15][0][0]=0x3fa;
        AC_matrix[15][0][1]=10;
        AC_matrix[15][1][0]=0x7fc3;
        AC_matrix[15][1][1]=16;
        AC_matrix[15][2][0]=0xfff6;
        AC_matrix[15][2][1]=16;
        AC_matrix[15][3][0]=0xfff7;
        AC_matrix[15][3][1]=16;
        AC_matrix[15][4][0]=0xfff8;
        AC_matrix[15][4][1]=16;
        AC_matrix[15][5][0]=0xfff9;
        AC_matrix[15][5][1]=16;
        AC_matrix[15][6][0]=0xfffa;
        AC_matrix[15][6][1]=16;
        AC_matrix[15][7][0]=0xfffb;
        AC_matrix[15][7][1]=16;
        AC_matrix[15][8][0]=0xfffc;
        AC_matrix[15][8][1]=16;
        AC_matrix[15][9][0]=0xfffd;
        AC_matrix[15][9][1]=16;
        AC_matrix[15][10][0]=0xfffe;
        AC_matrix[15][10][1]=16;
        }

        /*
        * INIT OF LUMINANCE
        */
        else
        {

        /*
        * init of the DC values for the luminance
        * [][0] is the code   [][1] is the number of bit
        */
        DC_matrix[0][0]=0x00;
        DC_matrix[0][1]=2;
        DC_matrix[1][0]=0x01;
        DC_matrix[1][1]=2;
        DC_matrix[2][0]=0x2;
        DC_matrix[2][1]=2;
        DC_matrix[3][0]=0x3;
        DC_matrix[3][1]=3;
        DC_matrix[4][0]=0xe;
        DC_matrix[4][1]=4;
        DC_matrix[5][0]=0x1e;
        DC_matrix[5][1]=5;
        DC_matrix[6][0]=0x3e;
        DC_matrix[6][1]=6;
        DC_matrix[7][0]=0x7e;
        DC_matrix[7][1]=7;
        DC_matrix[8][0]=0xfe;
        DC_matrix[8][1]=8;
        DC_matrix[9][0]=0x1fe;
        DC_matrix[9][1]=9;
        DC_matrix[10][0]=0x3fe;
        DC_matrix[10][1]=10;
        DC_matrix[11][0]=0x7fe;
        DC_matrix[11][1]=11;

        /*
        * Init of the AC hufmann code for luminance
        * matrix [][][0] is the code & matrix[][][1] is the number of bit
        */
        AC_matrix[0][0][0]=0xa;
        AC_matrix[0][0][1]=4;
        AC_matrix[0][1][0]=0x00;
        AC_matrix[0][1][1]=2;
        AC_matrix[0][2][0]=0x01;
        AC_matrix[0][2][1]=2;
        AC_matrix[0][3][0]=0x04;
        AC_matrix[0][3][1]=3;
        AC_matrix[0][4][0]=0xb;
        AC_matrix[0][4][1]=4;
        AC_matrix[0][5][0]=0x1a;
        AC_matrix[0][5][1]=5;
        AC_matrix[0][6][0]=0x78;
        AC_matrix[0][6][1]=7;
        AC_matrix[0][7][0]=0xf8;
        AC_matrix[0][7][1]=8;
        AC_matrix[0][8][0]=0x3f6;
        AC_matrix[0][8][1]=10;
        AC_matrix[0][9][0]=0xff82;
        AC_matrix[0][9][1]=16;
        AC_matrix[0][10][0]=0xff83;
        AC_matrix[0][10][1]=16;
        AC_matrix[1][1][0]=0xc;
        AC_matrix[1][1][1]=4;
        AC_matrix[1][2][0]=0x1b;
        AC_matrix[1][2][1]=5;
        AC_matrix[1][3][0]=0x79;
        AC_matrix[1][3][1]=7;
        AC_matrix[1][4][0]=0xf6;
        AC_matrix[1][4][1]=9;
        AC_matrix[1][5][0]=0x7f6;
        AC_matrix[1][5][1]=11;
        AC_matrix[1][6][0]=0xff84;
        AC_matrix[1][6][1]=16;
        AC_matrix[1][7][0]=0xff85;
        AC_matrix[1][7][1]=16;
        AC_matrix[1][8][0]=0xff86;
        AC_matrix[1][8][1]=16;
        AC_matrix[1][9][0]=0xff87;
        AC_matrix[1][9][1]=16;
        AC_matrix[1][10][0]=0xff88;
        AC_matrix[1][10][1]=16;
        AC_matrix[2][1][0]=0x1c;
        AC_matrix[2][1][1]=5;
        AC_matrix[2][2][0]=0xf9;
        AC_matrix[2][2][1]=8;
        AC_matrix[2][3][0]=0x3f7;
        AC_matrix[2][3][1]=10;
        AC_matrix[2][4][0]=0xff4;
        AC_matrix[2][4][1]=12;
        AC_matrix[2][5][0]=0xff89;
        AC_matrix[2][5][1]=16;
        AC_matrix[2][6][0]=0xff8a;
        AC_matrix[2][6][1]=16;
        AC_matrix[2][7][0]=0xff8b;
        AC_matrix[2][7][1]=16;
        AC_matrix[2][8][0]=0xff8c;
        AC_matrix[2][8][1]=16;
        AC_matrix[2][9][0]=0xff8d;
        AC_matrix[2][9][1]=16;
        AC_matrix[2][10][0]=0xff8e;
        AC_matrix[2][10][1]=16;
        AC_matrix[3][1][0]=0x3a;
        AC_matrix[3][1][1]=6;
        AC_matrix[3][2][0]=0x1f7;
        AC_matrix[3][2][1]=9;
        AC_matrix[3][3][0]=0xff5;
        AC_matrix[3][3][1]=12;
        AC_matrix[3][4][0]=0xff8f;
        AC_matrix[3][4][1]=16;
        AC_matrix[3][5][0]=0xff90;
        AC_matrix[3][5][1]=16;
        AC_matrix[3][6][0]=0xff91;
        AC_matrix[3][6][1]=16;
        AC_matrix[3][7][0]=0xff92;
        AC_matrix[3][7][1]=16;
        AC_matrix[3][8][0]=0xff93;
        AC_matrix[3][8][1]=16;
        AC_matrix[3][9][0]=0xff94;
        AC_matrix[3][9][1]=16;
        AC_matrix[3][10][0]=0xff95;
        AC_matrix[3][10][1]=16;
        AC_matrix[4][1][0]=0x3b;
        AC_matrix[4][1][1]=6;
        AC_matrix[4][2][0]=0x3f8;
        AC_matrix[4][2][1]=10;
        AC_matrix[4][3][0]=0xff96;
        AC_matrix[4][3][1]=16;
        AC_matrix[4][4][0]=0xff97;
        AC_matrix[4][4][1]=16;
        AC_matrix[4][5][0]=0xff98;
        AC_matrix[4][5][1]=16;
        AC_matrix[4][6][0]=0xff99;
        AC_matrix[4][6][1]=16;
        AC_matrix[4][7][0]=0xff9a;
        AC_matrix[4][7][1]=16;
        AC_matrix[4][8][0]=0xff9b;
        AC_matrix[4][8][1]=16;
        AC_matrix[4][9][0]=0xff9c;
        AC_matrix[4][9][1]=16;
        AC_matrix[4][10][0]=0xff9d;
        AC_matrix[4][10][1]=16;
        AC_matrix[5][1][0]=0x7a;
        AC_matrix[5][1][1]=7;
        AC_matrix[5][2][0]=0x7f7;
        AC_matrix[5][2][1]=11;
        AC_matrix[5][3][0]=0xff9e;
        AC_matrix[5][3][1]=16;
        AC_matrix[5][4][0]=0xff9f;
        AC_matrix[5][4][1]=16;
        AC_matrix[5][5][0]=0xffa0;
        AC_matrix[5][5][1]=16;
        AC_matrix[5][6][0]=0xffa1;
        AC_matrix[5][6][1]=16;
        AC_matrix[5][7][0]=0xffa2;
        AC_matrix[5][7][1]=16;
        AC_matrix[5][8][0]=0xffa3;
        AC_matrix[5][8][1]=16;
        AC_matrix[5][9][0]=0xffa4;
        AC_matrix[5][9][1]=16;
        AC_matrix[5][10][0]=0xffa5;
        AC_matrix[5][10][1]=16;
        AC_matrix[6][1][0]=0x7b;
        AC_matrix[6][1][1]=7;
        AC_matrix[6][2][0]=0xff6;
        AC_matrix[6][2][1]=12;
        AC_matrix[6][3][0]=0xffa6;
        AC_matrix[6][3][1]=16;
        AC_matrix[6][4][0]=0xffa7;
        AC_matrix[6][4][1]=16;
        AC_matrix[6][5][0]=0xffa8;
        AC_matrix[6][5][1]=16;
        AC_matrix[6][6][0]=0xffa9;
        AC_matrix[6][6][1]=16;
        AC_matrix[6][7][0]=0xffaa;
        AC_matrix[6][7][1]=16;
        AC_matrix[6][8][0]=0xffab;
        AC_matrix[6][8][1]=16;
        AC_matrix[6][9][0]=0xffac;
        AC_matrix[6][9][1]=16;
        AC_matrix[6][10][0]=0xffad;
        AC_matrix[6][10][1]=16;
        AC_matrix[7][1][0]=0xfa;
        AC_matrix[7][1][1]=8;
        AC_matrix[7][2][0]=0xff7;
        AC_matrix[7][2][1]=12;
        AC_matrix[7][3][0]=0xffae;
        AC_matrix[7][3][1]=16;
        AC_matrix[7][4][0]=0xffaf;
        AC_matrix[7][4][1]=16;
        AC_matrix[7][5][0]=0xffb0;
        AC_matrix[7][5][1]=16;
        AC_matrix[7][6][0]=0xffb1;
        AC_matrix[7][6][1]=16;
        AC_matrix[7][7][0]=0xffb2;
        AC_matrix[7][7][1]=16;
        AC_matrix[7][8][0]=0xffb3;
        AC_matrix[7][8][1]=16;
        AC_matrix[7][9][0]=0xffb4;
        AC_matrix[7][9][1]=16;
        AC_matrix[7][10][0]=0xffb5;
        AC_matrix[7][10][1]=16;
        AC_matrix[8][1][0]=0x1f8;
        AC_matrix[8][1][1]=9;
        AC_matrix[8][2][0]=0x7fa0;
        AC_matrix[8][2][1]=15;
        AC_matrix[8][3][0]=0xffb6;
        AC_matrix[8][3][1]=16;
        AC_matrix[8][4][0]=0xffb7;
        AC_matrix[8][4][1]=16;
        AC_matrix[8][5][0]=0xffb8;
        AC_matrix[8][5][1]=16;
        AC_matrix[8][6][0]=0xffb9;
        AC_matrix[8][6][1]=16;
        AC_matrix[8][7][0]=0xffba;
        AC_matrix[8][7][1]=16;
        AC_matrix[8][8][0]=0xffbb;
        AC_matrix[8][8][1]=16;
        AC_matrix[8][9][0]=0xffbc;
        AC_matrix[8][9][1]=16;
        AC_matrix[8][10][0]=0xffbd;
        AC_matrix[8][10][1]=16;
        AC_matrix[9][1][0]=0x1f9;
        AC_matrix[9][1][1]=9;
        AC_matrix[9][2][0]=0xffbe;
        AC_matrix[9][2][1]=16;
        AC_matrix[9][3][0]=0xffbf;
        AC_matrix[9][3][1]=16;
        AC_matrix[9][4][0]=0xffc0;
        AC_matrix[9][4][1]=16;
        AC_matrix[9][5][0]=0xffc1;
        AC_matrix[9][5][1]=16;
        AC_matrix[9][6][0]=0xffc2;
        AC_matrix[9][6][1]=16;
        AC_matrix[9][7][0]=0xffc3;
        AC_matrix[9][7][1]=16;
        AC_matrix[9][8][0]=0xffc4;
        AC_matrix[9][8][1]=16;
        AC_matrix[9][9][0]=0xffc5;
        AC_matrix[9][9][1]=16;
        AC_matrix[9][10][0]=0xffc6;
        AC_matrix[9][10][1]=16;
        AC_matrix[10][1][0]=0x1fa;
        AC_matrix[10][1][1]=9;
        AC_matrix[10][2][0]=0xffc7;
        AC_matrix[10][2][1]=16;
        AC_matrix[10][3][0]=0xffc8;
        AC_matrix[10][3][1]=16;
        AC_matrix[10][4][0]=0xffc9;
        AC_matrix[10][4][1]=16;
        AC_matrix[10][5][0]=0xffca;
        AC_matrix[10][5][1]=16;
        AC_matrix[10][6][0]=0xffcb;
        AC_matrix[10][6][1]=16;
        AC_matrix[10][7][0]=0xffcc;
        AC_matrix[10][7][1]=16;
        AC_matrix[10][8][0]=0xffcd;
        AC_matrix[10][8][1]=16;
        AC_matrix[10][9][0]=0xffce;
        AC_matrix[10][9][1]=16;
        AC_matrix[10][10][0]=0xffcf;
        AC_matrix[10][10][1]=16;
        AC_matrix[11][1][0]=0x3f9;
        AC_matrix[11][1][1]=10;
        AC_matrix[11][2][0]=0xffd0;
        AC_matrix[11][2][1]=16;
        AC_matrix[11][3][0]=0xffd1;
        AC_matrix[11][3][1]=16;
        AC_matrix[11][4][0]=0xffd2;
        AC_matrix[11][4][1]=16;
        AC_matrix[11][5][0]=0xffd3;
        AC_matrix[11][5][1]=16;
        AC_matrix[11][6][0]=0xffd4;
        AC_matrix[11][6][1]=16;
        AC_matrix[11][7][0]=0xffd5;
        AC_matrix[11][7][1]=16;
        AC_matrix[11][8][0]=0xffd6;
        AC_matrix[11][8][1]=16;
        AC_matrix[11][9][0]=0xffd7;
        AC_matrix[11][9][1]=16;
        AC_matrix[11][10][0]=0xffd8;
        AC_matrix[11][10][1]=16;
        AC_matrix[12][1][0]=0x3fa;
        AC_matrix[12][1][1]=10;
        AC_matrix[12][2][0]=0xffd9;
        AC_matrix[12][2][1]=16;
        AC_matrix[12][3][0]=0xffda;
        AC_matrix[12][3][1]=16;
        AC_matrix[12][4][0]=0xffdb;
        AC_matrix[12][4][1]=16;
        AC_matrix[12][5][0]=0xffdc;
        AC_matrix[12][5][1]=16;
        AC_matrix[12][6][0]=0xffdd;
        AC_matrix[12][6][1]=16;
        AC_matrix[12][7][0]=0xffde;
        AC_matrix[12][7][1]=16;
        AC_matrix[12][8][0]=0xffdf;
        AC_matrix[12][8][1]=16;
        AC_matrix[12][9][0]=0xffe0;
        AC_matrix[12][9][1]=16;
        AC_matrix[12][10][0]=0xffe1;
        AC_matrix[12][10][1]=16;
        AC_matrix[13][1][0]=0x7f8;
        AC_matrix[13][1][1]=11;
        AC_matrix[13][2][0]=0xffe2;
        AC_matrix[13][2][1]=16;
        AC_matrix[13][3][0]=0xffe3;
        AC_matrix[13][3][1]=16;
        AC_matrix[13][4][0]=0xffe4;
        AC_matrix[13][4][1]=16;
        AC_matrix[13][5][0]=0xffe5;
        AC_matrix[13][5][1]=16;
        AC_matrix[13][6][0]=0xffe6;
        AC_matrix[13][6][1]=16;
        AC_matrix[13][7][0]=0xffe7;
        AC_matrix[13][7][1]=16;
        AC_matrix[13][8][0]=0xffe8;
        AC_matrix[13][8][1]=16;
        AC_matrix[13][9][0]=0xffe9;
        AC_matrix[13][9][1]=16;
        AC_matrix[13][10][0]=0xffea;
        AC_matrix[13][10][1]=16;
        AC_matrix[14][1][0]=0xffeb;
        AC_matrix[14][1][1]=16;
        AC_matrix[14][2][0]=0xffec;
        AC_matrix[14][2][1]=16;
        AC_matrix[14][3][0]=0xffed;
        AC_matrix[14][3][1]=16;
        AC_matrix[14][4][0]=0xffee;
        AC_matrix[14][4][1]=16;
        AC_matrix[14][5][0]=0xffef;
        AC_matrix[14][5][1]=16;
        AC_matrix[14][6][0]=0xfff0;
        AC_matrix[14][6][1]=16;
        AC_matrix[14][7][0]=0xfff1;
        AC_matrix[14][7][1]=16;
        AC_matrix[14][8][0]=0xfff2;
        AC_matrix[14][8][1]=16;
        AC_matrix[14][9][0]=0xfff3;
        AC_matrix[14][9][1]=16;
        AC_matrix[14][10][0]=0xfff4;
        AC_matrix[14][10][1]=16;
        AC_matrix[15][0][0]=0x7f9;
        AC_matrix[15][0][1]=11;
        AC_matrix[15][1][0]=0xff5;
        AC_matrix[15][1][1]=16;
        AC_matrix[15][2][0]=0xfff6;
        AC_matrix[15][2][1]=16;
        AC_matrix[15][3][0]=0xfff7;
        AC_matrix[15][3][1]=16;
        AC_matrix[15][4][0]=0xfff8;
        AC_matrix[15][4][1]=16;
        AC_matrix[15][5][0]=0xfff9;
        AC_matrix[15][5][1]=16;
        AC_matrix[15][6][0]=0xfffa;
        AC_matrix[15][6][1]=16;
        AC_matrix[15][7][0]=0xfffb;
        AC_matrix[15][7][1]=16;
        AC_matrix[15][8][0]=0xfffc;
        AC_matrix[15][8][1]=16;
        AC_matrix[15][9][0]=0xfffd;
        AC_matrix[15][9][1]=16;
        AC_matrix[15][10][0]=0xfffe;
        AC_matrix[15][10][1]=16;
        }
    }

}
