
// Copyright (C) 1996 by Florian Raemy & LCAV, Swiss Federal Institute of 
// Technology, Lausanne, Switzerland.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//
// Visit Florian's Home Page http://studwww.eurecom.fr/~raemy and
// LCAV Department of EPFL: http://lcavwww.epfl.ch




import java.awt.*;
import java.io.*;
import java.util.*;
import java.lang.*;


/**
* Jpeg - The JPEG main program which perform a jpeg compression and decompression of an image.
* On the left image, there is the original. On the right, the Jpeg compressed image
**/

public class Jpeg extends Frame
{
    Image image;
    Grafixs GT;
    RLE Rle;
    Hufmann Huf;
    Display_Window iw;
    Stack RLE[];
    Stack HUF[];
    Stack IHUF[];
    int imageHeight, imageWidth;
    int Quality;
    int IRLE[][];
    int Red_image[][];
    int Blue_image[][];
    int Green_image[][];
    String code;
    int outputArray[][];
    int compImage[][];
    int one[];

    public Jpeg(int quality, String image_load,String code)
    {

        /*
        * Quality of the image.
        * 1 will gives you a good image with a low compression rate
        * 25-30 will gives you a poor quality image with a good compression rate
        */
        Quality=quality;

        /*
        * Load the image
        * !! Depending your OS and compiler, it can append that the image is not fully loaded
        */
        image = Toolkit.getDefaultToolkit().getImage(image_load);
        MediaTracker tracker = new MediaTracker(this);
        tracker.addImage(image, 0);
        System.out.println("Initialized tracker.");
        try
        {
            System.out.println("Waiting for tracker...");
            tracker.waitForID(0);
        }
        catch (Exception e) {return;}
        System.out.println("Loaded image.");

        /*
        * Getting picture information
        * It takes the Width, Height and RGB scans of the image. If it is not 8 multiples, the compressed image will have
        * black borders.
        */
        GT = new Grafixs(image);
        imageHeight=GT.imageHeight;
        imageWidth=GT.imageWidth;
        Rle = new RLE(imageWidth,imageHeight);

        /*
         * RGB COMPRESSION
         * In the main, if the RGB is selected, a RGB compression will be done.
         * It means that the quantized matrix will be different, as well as the Hufmann tables (not here but it should)
         */

        if (code.compareTo("RGB")==0)
        {

            Red_image= new int[imageWidth][imageHeight];
            Blue_image= new int[imageWidth][imageHeight];
            Green_image= new int[imageWidth][imageHeight];
            outputArray= new int[imageWidth][imageHeight];
            DCT_RGB dctTrans;

            /*
            * Load DCT for RGB
            */
            dctTrans = new DCT_RGB(Quality,imageWidth,imageHeight);

            /*
            * RED COMPONENT
            * First, the red component is loaded. The scan is the compressed, run lenght coded and hufmann coded.
            * Secondly, it is hufmann decoded, run lenght decoded and decompressed.
            */
            outputArray=GT.getRedArray();
            compImage=dctTrans.compressImage(outputArray,imageWidth,imageHeight);
            RLE=Rle.RLE(compImage);
            Huf=new Hufmann(0,imageWidth,imageHeight);
            HUF=Huf.Hufmann_encode(RLE);
            IHUF=Huf.Hufmann_decode(HUF);
            IRLE=Rle.IRLE(IHUF);
            Red_image=dctTrans.decompressImage(IRLE,imageWidth,imageHeight);

            /*
            * BLUE COMPONENT
            * First, the red component is loaded. The scan is the compressed, run lenght coded and hufmann coded.
            * Secondly, it is hufmann decoded, run lenght decoded and decompressed.
            */
            outputArray=GT.getBlueArray();
            compImage=dctTrans.compressImage(outputArray,imageWidth,imageHeight);
            RLE=Rle.RLE(compImage);
            Huf=new Hufmann(0,imageWidth,imageHeight);
            HUF=Huf.Hufmann_encode(RLE);
            IHUF=Huf.Hufmann_decode(HUF);
            IRLE=Rle.IRLE(IHUF);
            Blue_image=dctTrans.decompressImage(IRLE,imageWidth,imageHeight);

            /*
            * GREEN COMPONENT
            * First, the red component is loaded. The scan is the compressed, run lenght coded and hufmann coded.
            * Secondly, it is hufmann decoded, run lenght decoded and decompressed.
            */
            outputArray=GT.getGreenArray();
            compImage=dctTrans.compressImage(outputArray,imageWidth,imageHeight);
            RLE=Rle.RLE(compImage);
            Huf=new Hufmann(0,imageWidth,imageHeight);
            HUF=Huf.Hufmann_encode(RLE);
            IHUF=Huf.Hufmann_decode(HUF);
            IRLE=Rle.IRLE(IHUF);
            Green_image=dctTrans.decompressImage(IRLE,imageWidth,imageHeight);


            /*
            * Conversion from RGB to Array
            */
            System.out.println("Constructing the image converting RGB to Array ..");
            one = new int[imageHeight*imageWidth];
            one = GT.convertRGBtoArray(Red_image,Green_image,Blue_image);
        }


        /*
         * YUV COMPRESSION
         * In the main, if the YUV is selected, a YCrCb compression will be done.
         * It means that the quantized matrix will be different, as well as the Hufmann tables
         */
        if (code.compareTo("YUV")==0)
        {
            int Yn_image1[][];
            int Cb_image1[][];
            int Cr_image1[][];
            float Yn_image2[][];
            float Cb_image2[][];
            float Cr_image2[][];
            DCT_YUV dctTrans;
            Red_image= new int[imageWidth][imageHeight];
            Blue_image= new int[imageWidth][imageHeight];
            Green_image= new int[imageWidth][imageHeight];
            Yn_image2= new float[imageWidth][imageHeight];
            Cr_image2= new float[imageWidth][imageHeight];
            Cb_image2= new float[imageWidth][imageHeight];
            outputArray= new int[imageWidth][imageHeight];
            compImage= new int[imageWidth][imageHeight];

            /*
            * Load DCT for YCrCb
            */
            dctTrans = new DCT_YUV(Quality,imageWidth,imageHeight);

            /*
            * Get the RGB Components
            */
            Red_image=GT.getRedArray();
            Blue_image=GT.getBlueArray();
            Green_image=GT.getGreenArray();

            /*
            * Convert the RGB to YCrCb
            */
            Yn_image2=GT.convertRGBtoY(Red_image,Green_image,Blue_image);
            Cr_image2=GT.convertRGBtoCr(Red_image,Green_image,Blue_image);
            Cb_image2=GT.convertRGBtoCb(Red_image,Green_image,Blue_image);

            /*
            * Y COMPONENT
            * The scan is the compressed, run lenght coded and hufmann coded.
            * Secondly, it is hufmann decoded, run lenght decoded and decompressed.
            */
            System.out.println("Y component");
            compImage=dctTrans.compressImage(Yn_image2,imageWidth,imageHeight,1);
            RLE=Rle.RLE(compImage);
            Huf=new Hufmann(1,imageWidth,imageHeight);
            HUF=Huf.Hufmann_encode(RLE);
            IHUF=Huf.Hufmann_decode(HUF);
            IRLE=Rle.IRLE(IHUF);
            Yn_image1=dctTrans.decompressImage(IRLE,imageWidth,imageHeight,1);

            /*
            * Cb COMPONENT
            * The scan is the compressed, run lenght coded and hufmann coded.
            * Secondly, it is hufmann decoded, run lenght decoded and decompressed.
            */
            System.out.println("Cb component");
            compImage=dctTrans.compressImage(Cb_image2,imageWidth,imageHeight,0);
            RLE=Rle.RLE(compImage);
            Huf=new Hufmann(0,imageWidth,imageHeight);
            HUF=Huf.Hufmann_encode(RLE);
            IHUF=Huf.Hufmann_decode(HUF);
            IRLE=Rle.IRLE(IHUF);
            Cb_image1=dctTrans.decompressImage(IRLE,imageWidth,imageHeight,0);

            /*
            * Cr COMPONENT
            * The scan is the compressed, run lenght coded and hufmann coded.
            * Secondly, it is hufmann decoded, run lenght decoded and decompressed.
            */
            System.out.println("Cr component");
            compImage=dctTrans.compressImage(Cr_image2,imageWidth,imageHeight,0);
            RLE=Rle.RLE(compImage);
            Huf=new Hufmann(0,imageWidth,imageHeight);
            HUF=Huf.Hufmann_encode(RLE);
            IHUF=Huf.Hufmann_decode(HUF);
            IRLE=Rle.IRLE(IHUF);
            Cr_image1=dctTrans.decompressImage(IRLE,imageWidth,imageHeight,0);

            /*
            * Convert the YUV to RGB
            */

            System.out.println("Convert YCbCr to RGB");
            Red_image=GT.convertYCbCrtoR(Yn_image1,  Cb_image1, Cr_image1);
            Blue_image=GT.convertYCbCrtoB(Yn_image1, Cb_image1, Cr_image1);
            Green_image=GT.convertYCbCrtoG(Yn_image1, Cb_image1, Cr_image1);

            /*
            * Conversion from RGB to Array
            */
            System.out.println("Constructing the image converting RGB to Array ..");
            one= new int[imageHeight*imageWidth];
            one = GT.convertRGBtoArray(Red_image,Green_image,Blue_image);
        }

        /*
         * BW COMPRESSION
         * In the main, if the BW is selected, a Black & White compression will be done.
         * It means that the quantized matrix will be different, as well as the Hufmann tables
         */
        if (code.compareTo("BW")==0)
        {
            int [][] Gray_image;
            DCT_RGB dctTrans;
            Gray_image= new int[imageWidth][imageHeight];
            outputArray= new int[imageWidth][imageHeight];
            compImage= new int[imageWidth][imageHeight];

            /*
            * Load DCT for RGB
            */
            dctTrans = new DCT_RGB(Quality,imageWidth,imageHeight);

            /*
            * GRAY COMPONENT
            * The scan is the compressed, run lenght coded and hufmann coded.
            * Secondly, it is hufmann decoded, run lenght decoded and decompressed.
            */
            outputArray=GT.getRedArray();
            compImage=dctTrans.compressImage(outputArray,imageWidth,imageHeight);
            RLE=Rle.RLE(compImage);
            Huf=new Hufmann(0,imageWidth,imageHeight);
            HUF=Huf.Hufmann_encode(RLE);
            IHUF=Huf.Hufmann_decode(HUF);
            IRLE=Rle.IRLE(IHUF);
            Gray_image=dctTrans.decompressImage(IRLE,imageWidth,imageHeight);

            /*
             * CONVERSION FROM ARRAY TO GRAY
             */
            System.out.println("Constructing the image converting Gray to Array ..");
            one = new int[imageHeight*imageWidth];
            one = GT.convertGrayToArray(Gray_image);
        }


        /*
        * Display the Jpeg image in a window, next to the original
        */
        Window iw = new Display_Window(one,imageWidth,imageHeight,image_load);
	iw.show();
        /*
        * You so have all the time you need to see this wonderful result
        */
        System.out.println("Hit enter ...");
        try {System.in.read();} catch (Exception e) {}
        System.exit(0);
    }



    /************** Main Method ****************/
    /***
    * You just need to annonce the name of the image, which should be on the same directory
    * as your program.
    * You can choose the quality too and the way of compressing the image ("RGB" or "YUV")
    * Jpeg(Quality, "Imagefile","Compression Type")
    * According to JAVA virtual machine, the files which can be readed are jpeg, tiff and gif files
    ***/

    public static void main(String args[])
    {
      if (args.length == 3) {
	try
	  {
	    new Jpeg(Integer.parseInt(args[0]),args[1],args[2]);
	  }
        catch (Exception e) {
	}
      }
      else {
	System.out.println("Uso: Jpeg calidad fichero.gif|.jpeg|.tiff RGB|YUV");
      }
    }
}
